# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# page.py
#
# Surclasse de l'objet Page woven pour gestion de l'authentification,
# des sessions et de la connexion au back-end
#       
###########################################################################
import twisted
## Eole 2
try:
    assert twisted.__version__[0] == '2'
    from twisted.web import http
except:
    from twisted.protocols import http
from twisted.web import error
from twisted.python import components
from twisted.web import xmlrpc,server
from twisted.web.resource import Resource
from zope.interface import Interface
from zephir.web import config
from zephir.eolerpclib import xmlrpclib
import ldap

# définition des classes servant a créer de nouvelles variables dans les sessions
class IPreferences(Interface):
    pass

class Preferences(components.Adapter):
    __implements__ = IPreferences

    def __init__(self,original):
        self.authentified = 'no'
        self.bad_auth = '0'
        self.password = ""
        self.user = ""
        components.Adapter.__init__(self,original)
    

components.registerAdapter(Preferences, server.Session, IPreferences)

def proxy(request):
    prefs = request.getSession(IPreferences)
    # récupération du proxy vers le backend zephir
    return getattr(prefs, 'zephir')


# ressource sécurisée
class PageEole(Resource):
 
    def __init__(self, *args, **kwargs):

        self.client_pass = None
 
        self.zephir=None
 
        self.serveur_ldap = config.ADRESSE_LDAP
        # config.SERVEUR_LDAP
        Resource.__init__(self, *args, **kwargs)
        
    def close_session(self,request):
        # on renvoie sur la page de déconnexion
        prefs = request.getSession(IPreferences)
        setattr(prefs,'authentified','no')
        setattr(prefs,'bad_auth','0')
        setattr(prefs,'user','')
        setattr(prefs,'password','')
        setattr(prefs,'zephir',None)
        if hasattr(prefs,'dico_eole'):
            delattr(prefs,'dico_eole')
        request.setHeader('WWW-authenticate', 'Basic realm="%s"' % ("zephir",))
        # request.path=request.prePathURL() + '/logout'

    def require_auth(self,request):
        prefs = request.getSession(IPreferences)
        request.setResponseCode(http.UNAUTHORIZED)
        request.setHeader('WWW-authenticate', 'Basic realm="%s"' % ("zephir",))
        errpage = error.ErrorPage(http.UNAUTHORIZED,"Unauthorized","401 Authentication required")
        bad_auth = getattr(prefs, 'bad_auth')
        # on incrémente bad_auth pour savoir combien de demandes d'autorisation on a fait
        setattr(prefs,'bad_auth',str(int(bad_auth)+1))
        return errpage.render(request)

    def render(self,request):
        request.setHeader('WWW-authenticate', 'Basic realm="%s"' % ("zephir-parc",))
        # test de l'authentification
        prefs = request.getSession(IPreferences)
        # on regarde si la session est authentifiée
        auth_ok = getattr(prefs, 'authentified')
        cred_user = getattr(prefs, 'user')
        cred_password = getattr(prefs, 'password')
        # bad_auth : permet de savoir quand une tentative de login a echoué
        bad_auth = getattr(prefs, 'bad_auth')
        # retries : permet de forcer le nombre de demandes d'autorisations a
        # envoyer pour que le navigateur demande un nouveau login
        # (ex : 2 pour Galeon, 1 pour IE, ...)
        retries = 1
        if auth_ok == 'no' or cred_user == '':
            # une fois le nombre voulu de demandes envoyées, on récupère les nouveaus login/password
            if int(bad_auth) >= retries:
                cred_user = request.getUser()
                cred_password = request.getPassword()
                # on vérifie l'authentification dans l'annuaire LDAP
                if cred_password == "" or cred_user == "":
                    return self.require_auth(request)
                try:
                    query=ldap.open(self.serveur_ldap)
                    # on récupère le dn complet de l'utilisateur (l'uid et le dn doivent être accessibles en lecture)
                    result=query.search_s(config.BASE_LDAP, ldap.SCOPE_SUBTREE, "(uid="+cred_user+")")
                    cred_dn = result[0][0]
                    # on tente une connexion à l'annuaire avec les informations données
                    query.simple_bind_s(cred_dn,cred_password)
                    query.unbind()
                except:
                    # erreur d'authentification
                    # on retourne un message d'erreur
                    print "authentification incorrecte : ",request.host.host
                else:
                    # on enregistre la session
                    # on lance la procédure de vérification de l'expiration de la session
                    request.getSession().notifyOnExpire(lambda :self.close_session(request))
                    request.getSession().checkExpired()
                    # création du serveur xmlrpc
                    # on modifie les préférences de la session pour indiquer qu'elle est authentifiée
                    setattr(prefs,'authentified','yes')
                    setattr(prefs,'bad_auth','0')
                    setattr(prefs,'user',cred_user)
                    setattr(prefs,'password',cred_password)
                    setattr(prefs,'zephir', xmlrpclib.ServerProxy("http://"+request.getUser()+":"+request.getPassword()+"@localhost:"+str(int(config.PORT_ZEPHIR) + 1) + "/"))
                    return self.render_design(request)
            return self.require_auth(request)
        else:
            request.getSession().touch()
            # l'authentification a réussi, on retourne le rendu de la page
            return self.render_design(request)

